"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
/*tslint:disable*/
const vscode_css_languageservice_1 = require("vscode-css-languageservice");
const vscode_languageserver_1 = require("vscode-languageserver");
const languageModelCache_1 = require("./languageModelCache");
const convert_1 = require("./utils/convert");
const runner_1 = require("./utils/runner");
// Create a connection for the server.
const connection = vscode_languageserver_1.createConnection();
console.log = connection.console.log.bind(connection.console);
console.error = connection.console.error.bind(connection.console);
process.on('unhandledRejection', (e) => {
    connection.console.error(runner_1.formatError(`Unhandled exception`, e));
});
// Create a simple text document manager. The text document manager
// supports full document sync only
const documents = new vscode_languageserver_1.TextDocuments();
// Make the text document manager listen on the connection
// for open, change and close text document events
documents.listen(connection);
const stylesheets = languageModelCache_1.getLanguageModelCache(10, 60, document => getLanguageService(document).parseStylesheet(document));
documents.onDidClose(e => {
    stylesheets.onDocumentRemoved(e.document);
});
connection.onShutdown(() => {
    stylesheets.dispose();
});
let foldingRangeLimit = Number.MAX_VALUE;
// After the server has started the client sends an initialize request. The server receives
// in the passed params the rootPath of the workspace plus the client capabilities.
connection.onInitialize((params) => {
    function getClientCapability(name, def) {
        const keys = name.split('.');
        let c = params.capabilities;
        for (let i = 0; c && i < keys.length; i++) {
            if (!c.hasOwnProperty(keys[i])) {
                return def;
            }
            c = c[keys[i]];
        }
        return c;
    }
    const snippetSupport = !!getClientCapability('textDocument.completion.completionItem.snippetSupport', false);
    foldingRangeLimit = getClientCapability('textDocument.foldingRange.rangeLimit', 1000);
    const capabilities = {
        // Tell the client that the server works in FULL text document sync mode
        textDocumentSync: documents.syncKind,
        completionProvider: snippetSupport
            ? { resolveProvider: false, triggerCharacters: [':', '.'] }
            : undefined,
        hoverProvider: true,
        documentSymbolProvider: true,
        referencesProvider: true,
        definitionProvider: true,
        documentHighlightProvider: true,
        codeActionProvider: true,
        renameProvider: true,
        colorProvider: {},
        foldingRangeProvider: true
    };
    return { capabilities };
});
const languageServices = {
    css: vscode_css_languageservice_1.getCSSLanguageService(),
    scss: vscode_css_languageservice_1.getSCSSLanguageService(),
    less: vscode_css_languageservice_1.getLESSLanguageService(),
    wxss: vscode_css_languageservice_1.getCSSLanguageService()
};
function getLanguageService(document) {
    let service = languageServices[document.languageId];
    if (!service) {
        connection.console.log('Document type is ' + document.languageId + ', using css instead.');
        service = languageServices['css'];
    }
    return service;
}
let documentSettings;
// The settings have changed. Is send on server activation as well.
connection.onDidChangeConfiguration(change => {
    let settings = change.settings;
    updateConfiguration(settings);
});
function updateConfiguration(settings) {
    for (const languageId in languageServices) {
        languageServices[languageId].configure(settings[languageId]);
    }
    // reset all document settings
    documentSettings = Object.assign({}, settings);
    // Revalidate any open text documents
    documents.all().forEach(triggerValidation);
}
const pendingValidationRequests = {};
const validationDelayMs = 200;
// The content of a text document has changed. This event is emitted
// when the text document first opened or when its content has changed.
documents.onDidChangeContent(change => {
    triggerValidation(change.document);
});
// a document has closed: clear all diagnostics
documents.onDidClose(event => {
    cleanPendingValidation(event.document);
    connection.sendDiagnostics({ uri: event.document.uri, diagnostics: [] });
});
function cleanPendingValidation(textDocument) {
    const request = pendingValidationRequests[textDocument.uri];
    if (request) {
        clearTimeout(request);
        delete pendingValidationRequests[textDocument.uri];
    }
}
function triggerValidation(textDocument) {
    cleanPendingValidation(textDocument);
    pendingValidationRequests[textDocument.uri] = setTimeout(() => {
        delete pendingValidationRequests[textDocument.uri];
        validateTextDocument(textDocument);
    }, validationDelayMs);
}
function validateTextDocument(textDocument) {
    try {
        const stylesheet = stylesheets.get(textDocument);
        const diagnostics = getLanguageService(textDocument).doValidation(textDocument, stylesheet, documentSettings[textDocument.languageId]);
        // Send the computed diagnostics to VSCode.
        connection.sendDiagnostics({ uri: textDocument.uri, diagnostics });
    }
    catch (e) {
        connection.console.error(runner_1.formatError(`Error while validating ${textDocument.uri}`, e));
    }
}
connection.onCompletion((textDocumentPosition, token) => {
    return runner_1.runSafe(() => {
        const document = documents.get(textDocumentPosition.textDocument.uri);
        if (!document) {
            return null;
        }
        const cssLS = getLanguageService(document);
        const result = cssLS.doComplete(document, textDocumentPosition.position, stylesheets.get(document));
        return {
            isIncomplete: false,
            items: convert_1.convertCompleteItems(result.items)
        };
    }, null, `Error while computing completions for ${textDocumentPosition.textDocument.uri}`, token);
});
connection.onHover((textDocumentPosition, token) => {
    return runner_1.runSafe(() => {
        const document = documents.get(textDocumentPosition.textDocument.uri);
        if (document) {
            const styleSheet = stylesheets.get(document);
            return getLanguageService(document).doHover(document, textDocumentPosition.position, styleSheet);
        }
        return null;
    }, null, `Error while computing hover for ${textDocumentPosition.textDocument.uri}`, token);
});
connection.onDocumentSymbol((documentSymbolParams, token) => {
    return runner_1.runSafe(() => {
        const document = documents.get(documentSymbolParams.textDocument.uri);
        if (document) {
            const stylesheet = stylesheets.get(document);
            return getLanguageService(document).findDocumentSymbols(document, stylesheet);
        }
        return [];
    }, [], `Error while computing document symbols for ${documentSymbolParams.textDocument.uri}`, token);
});
connection.onDefinition((documentSymbolParams, token) => {
    return runner_1.runSafe(() => {
        const document = documents.get(documentSymbolParams.textDocument.uri);
        if (document) {
            const stylesheet = stylesheets.get(document);
            return getLanguageService(document).findDefinition(document, documentSymbolParams.position, stylesheet);
        }
        return null;
    }, null, `Error while computing definitions for ${documentSymbolParams.textDocument.uri}`, token);
});
connection.onDocumentHighlight((documentSymbolParams, token) => {
    return runner_1.runSafe(() => {
        const document = documents.get(documentSymbolParams.textDocument.uri);
        if (document) {
            const stylesheet = stylesheets.get(document);
            return getLanguageService(document).findDocumentHighlights(document, documentSymbolParams.position, stylesheet);
        }
        return [];
    }, [], `Error while computing document highlights for ${documentSymbolParams.textDocument.uri}`, token);
});
connection.onReferences((referenceParams, token) => {
    return runner_1.runSafe(() => {
        const document = documents.get(referenceParams.textDocument.uri);
        if (document) {
            const stylesheet = stylesheets.get(document);
            return getLanguageService(document).findReferences(document, referenceParams.position, stylesheet);
        }
        return [];
    }, [], `Error while computing references for ${referenceParams.textDocument.uri}`, token);
});
connection.onCodeAction((codeActionParams, token) => {
    return runner_1.runSafe(() => {
        const document = documents.get(codeActionParams.textDocument.uri);
        if (document) {
            const stylesheet = stylesheets.get(document);
            return getLanguageService(document).doCodeActions(document, codeActionParams.range, codeActionParams.context, stylesheet);
        }
        return [];
    }, [], `Error while computing code actions for ${codeActionParams.textDocument.uri}`, token);
});
connection.onDocumentColor((params, token) => {
    return runner_1.runSafe(() => {
        const document = documents.get(params.textDocument.uri);
        if (document) {
            const stylesheet = stylesheets.get(document);
            return getLanguageService(document).findDocumentColors(document, stylesheet);
        }
        return [];
    }, [], `Error while computing document colors for ${params.textDocument.uri}`, token);
});
connection.onColorPresentation((params, token) => {
    return runner_1.runSafe(() => {
        const document = documents.get(params.textDocument.uri);
        if (document) {
            const stylesheet = stylesheets.get(document);
            return getLanguageService(document).getColorPresentations(document, stylesheet, params.color, params.range);
        }
        return [];
    }, [], `Error while computing color presentations for ${params.textDocument.uri}`, token);
});
connection.onRenameRequest((renameParameters, token) => {
    return runner_1.runSafe(() => {
        const document = documents.get(renameParameters.textDocument.uri);
        if (document) {
            const stylesheet = stylesheets.get(document);
            return getLanguageService(document).doRename(document, renameParameters.position, renameParameters.newName, stylesheet);
        }
        return null;
    }, null, `Error while computing renames for ${renameParameters.textDocument.uri}`, token);
});
connection.onFoldingRanges((params, token) => {
    return runner_1.runSafe(() => {
        const document = documents.get(params.textDocument.uri);
        if (document) {
            return getLanguageService(document).getFoldingRanges(document, {
                rangeLimit: foldingRangeLimit
            });
        }
        return null;
    }, null, `Error while computing folding ranges for ${params.textDocument.uri}`, token);
});
// Listen on the connection
connection.listen();
//# sourceMappingURL=server.js.map